@@ -1,4 +1,4 @@
-//  $Id: rawhoisc.cc,v 4.11 2006/06/08 15:16:26 shane Exp $
+//  $Id: rawhoisc.cc,v 4.14 2007/02/13 00:25:01 shane Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -150,7 +150,6 @@ void RAWhoisClient::Open(const char *_host, const int _port, const char *_source
 	       sizeof(server_sockaddr)) < 0)
       error.Die("Error: connect() failed.\n");
 
-
    in  = fdopen(sock, "r");
    out = fdopen(sock, "w");
    _is_open = 1;
@@ -187,7 +186,7 @@ void RAWhoisClient::CheckConnection() {
 }
 
 void RAWhoisClient::GetVersion() {
-   char *buffer = (char *) calloc (80,1);
+   char *buffer;
    char *start;
 
    if (! _is_open)
@@ -199,7 +198,7 @@ void RAWhoisClient::GetVersion() {
    start = start + 8; //jump
    version = atoi(start)*10 + atoi(start+2); // x.x... format  
    Trace(TR_WHOIS_RESPONSE) << "Whois: Response " << buffer << endl;
-   free(buffer);
+   delete [] buffer;
 }
 
 void RAWhoisClient::SetSources(const char *_sources) {
@@ -297,36 +296,57 @@ int RAWhoisClient::Response(char *&response) {
       error.Die("Warning: no byte count error for query %s.\n", last_query);
       return 0;
    }
-   if (is_rpslng()) {
-      response = strdup("");
-      char *prev;
+   /*
+    * If we sent an RIPE-style query (as we do for *some* RPSLng queries), 
+    * then we want to parse it using RIPE-style handling.
+    *
+    * But we will attempt to detect use of IRRd queries. We assume that 
+    * RIPE-style queries start with either an error (meaning the first 
+    * character is '%') or they start with an attribute name (meaning 
+    * there is a ':' somewhere in the buffer).
+    */
+   if (is_rpslng() && 
+       ((buffer[0] == '%') || (strchr(buffer, ':') != NULL))) {
+      int wasnl = 0;
+      response = new char[1];
+      response[0] = '\0';
+
       do {
-        prev = strdup(buffer);
-        Trace(TR_WHOIS_RESPONSE) << "Whois: Response <<\n" << buffer <<">>"<< endl;
+        wasnl = (*buffer == '\n');
+        Trace(TR_WHOIS_RESPONSE) << "Whois: Response <<\n" << buffer
+                                 << ">>" << endl;
         if (strstr (buffer, "route") || strstr(buffer, "route6")) {
-          char *prefix;
-          char end_prefix[18];
-          char *tmp;
-          prefix = strstr(buffer, ":");
+          char *p = strstr(buffer, ":");
           do {
-            prefix++;
-          } while (isspace (*prefix));
-          sscanf (prefix, "%s\n", &end_prefix);
-          // save response
-          tmp = strdup (response);
-          // allocate new string
-          response = new char [strlen(tmp) + strlen(end_prefix) + 2];
-          memset(response, 0, strlen(response));
-          // copy old and new response
-          strncat (response, tmp, strlen(tmp));
-          strncat (response, " ", 1);
-          strncat (response, end_prefix, strlen(end_prefix));
-          free(tmp);
+            p++;
+          } while (isspace (*p));
+
+          // strip trailing newline
+          int len = strlen (p);
+          if(len > 0) {
+            char *q = p + len - 1;
+            if(*q == '\n')
+                *q = '\0';
+            len--;
+          }
+
+          // save current response
+          len += strlen (response) + 2;
+          char *copy = strdup (response);
+
+          // reallocate response with more space
+          delete [] response;
+          response = new char [len];
+
+          // append new repsonse onto old one
+          snprintf (response, len, "%s %s", copy, p);
+          free (copy);
         }
-      } while (fgets(buffer, sizeof(buffer), in) && 
+      } while (fgets(buffer, sizeof(buffer), in) &&
       // this condition should work with irrd version >= 2.2b19
-      // until then, ripe-style queries won't work with persistent connections
-               !((*prev == '\n') && (*buffer == '\n')));
+      // until then, ripe-style queries won't work with persistent
+      // connections
+               !(wasnl && (*buffer == '\n')));
 
       // The WHOIS protocol and RPSL give no indication of
       // end of a protocol data unit, so we need to keep
