@@ -1,4 +1,4 @@
-//  $Id: RadixSet.cc,v 4.10 2006/06/08 15:16:26 shane Exp $
+//  $Id: RadixSet.cc,v 4.11 2007/02/09 19:46:31 shane Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -878,7 +878,7 @@ ostream& operator<<(ostream& o, const RadixSet &set) {
       	       need_comma = true;
       	    o << int2quad(buffer, addr) << "/" << leng << "^" << n << "-" << m;
       	 } // end of for loop
-      } else { // not a compressed point
+      } else { // not a compressed print
        	 RadixSet::PrefixIterator itr(&set);
 	       for (bool flag = itr.first(addr, leng); flag; flag = itr.next(addr, leng)) {
            if (need_comma)
