@@ -1,4 +1,4 @@
-//  $Id: prefix.cc,v 4.11 2006/10/01 11:23:32 shane Exp $
+//  $Id: prefix.cc,v 4.12 2007/02/09 19:46:31 shane Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -1216,10 +1216,13 @@ void MPPrefixRanges::except(MPPrefixRanges *src) {
    assign(result.begin(), result.end());
 }
 
+// contains methods rewritten by spz according to hint from Hagen Boehm
 bool MPPrefixRanges::contains(IPAddr ip) const {
    MPPrefixRanges::const_iterator p;
    for (p = begin(); p != end(); ++p) {
-     if (p->ipv4 && (p->ipv4->get_ipaddr() == ip.get_ipaddr())) 
+     if (p->ipv4
+		&& p->ipv4->get_ipaddr() == ip.get_ipaddr()
+		&& p->ipv4->get_length() == ip.get_length() )
        return true;
    }
    return false;
@@ -1228,7 +1231,9 @@ bool MPPrefixRanges::contains(IPAddr ip) const {
 bool MPPrefixRanges::contains(IPv6Addr ip) const {
    MPPrefixRanges::const_iterator p;
    for (p = begin(); p != end(); ++p) {
-     if (p->ipv6 && (*(p->ipv6->get_ipaddr()) == *(ip.get_ipaddr())) )
+     if (p->ipv6
+		&& p->ipv6->get_ipaddr() == ip.get_ipaddr()
+		&& p->ipv6->get_length() == ip.get_length() )
        return true;
    }
    return false;
